
; 2 DO
; nnu snabbare I2C?
; Stda upp Delayloopar nr jag har en 16F84A (20Mhz)



;****************************************************************************             
;*** NIXIE.ASM   6x IN-18 Tubes
;*** 
;*** Author: (C)-2002 Christian Mohr // Christian@Mohr.se
;*** 
;*** May not be published in any form without permission from the author!
;****************************************************************************            
;*** CPU: 16C84
;*** CLOCK: !!!16MhZ Xtal!!!
;****************************************************************************            


; MAIN LOOP MODES
; 00 = DISPLAY TIME
; 01 = SET 10 x Hour
; 02 = SET  1 x Hour
; 03 = SET 10 x Minute
; 04 = SET  1 x Minute
; 05 = SET 10 x Second
; 06 = SET  1 x Second

; CATHODE SELECT BITS TO SHREG - First bit = 5x   Last Bit = x0
; DIGIT DISPLAYED:  5x,4x,3x,2x,1x,0x,x9,x8,x7,x6,x5,x4,x3,x2,x1,x0
; CLOCKCYCLE:       01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16
;
; COMMON ANODE SELECT
; X0=1 --> SECOND DIGITS ON
; X1=1 --> MINUTE DIGITS ON
; X2=1 --> HOUR DIGITS ON


        Radix   DEC
        EXPAND

        include "p16c84.inc"


        __CONFIG 0x09




;****************************************************************************             
;*** PORT DECLARATIONS
;****************************************************************************            

#define NCK	PORTA,0		; NIXIE CLOCK
#define NEN	PORTA,1		; NIXIE ENABLE
#define NDA	PORTA,2		; NIXIE DATA
#define NST	PORTA,3		; NIXIE STROBE

#define I2C_C	PORTB,0		; I2C Clock
#define I2C_D	PORTB,1		; I2C Data
#define X2	PORTB,2		; HH Digits Select
#define X1	PORTB,3		; MM Digits Select
#define X0	PORTB,4		; SS Digits Select
#define B_MNU	PORTB,5		; BUTTON 0 = MENU/OK
#define B_PL	PORTB,6		; BUTTON 1 = +
#define B_MI	PORTB,7		; BUTTON 2 = -

;****************************************************************************             
;*** RAM DECLARATIONS
;****************************************************************************            


BDEB		set	0x1F	; BUTTON DEBOUNCE
MODE		set	0x20	; NORMAL/SET MODE FLAG'S
MPX		set	0x21	; MULTIPLEX POINTER
FLASH		set	0x22    ; CURSOR[5..0] = FLASHES DIGIT
FLCNT		set	0x23	; FLASH COUNTER
IOBYTE  	set     0x24	; I/O DATA STORAGE
H1		set	0x25	; HOURS 1	; DO NOT MOVE
H0		set	0x26	; HOURS 10	; DO NOT MOVE
M1		set	0x27	; MINUTES 1	; DO NOT MOVE
M0		set	0x28	; MINUTES 10	; DO NOT MOVE
S1		set	0x29	; SECONDS 1	; DO NOT MOVE
S0		set	0x2A	; SECONDS 10	; DO NOT MOVE
_N		set	0x2B	; BITCOUNTER
T3		set	0x2C	; TEMP. STORAGE
T2		set	0x2D	; TEMP. STORAGE
T1		set	0x2E	; TEMP. STORAGE
T0		set	0x2F	; TEMP. STORAGE


;****************************************************************************             
;*** Vectors
;****************************************************************************            

        ORG     0x00
	goto	Start
        ORG     0x04
	goto	Interrupt

; ******************************************************************
; *** SET PORTS TO INPUT / OUTPUT AND INITIALIZE VARIABLES
; ******************************************************************
INIT:	bsf     STATUS,RP0     ; Select Page 1 for Tris access

	bcf	NEN
	bcf	NDA
	bcf	NST
	bcf	NCK
	bcf	I2C_C
	bcf	X0
	bcf	X1
	bcf	X2
	bsf	B_MNU
	bsf	B_PL
	bsf	B_MI
	bcf	OPTION_REG,7
        bcf     STATUS,RP0     ; select Page 0 for Port Access

	bcf	NEN
	bcf	NCK
	bcf	NST
	bsf	B_MNU
	bsf	B_PL
	bsf	B_MI
	bcf	X0
	bcf	X1
	bcf	X2

        RETURN

; ******************************************************************
; *** ISR
; ******************************************************************

Interrupt:

	RETFIE



; ******************************************************************
; *** WRITE TIME TO RTC
; ******************************************************************

WTIME:	CALL    I2CSTA
        MOVLW   0xA0
        CALL    I2CWB
        MOVLW   0x01
        CALL    I2CWB
        MOVLW   0x00    ; 100/s SECONDS = 00
	CALL    I2CWB

        MOVF    S1,W    ; SECONDS
        MOVWF   T0
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        MOVLW   0xF0
        ANDWF   T0,F
        MOVLW   0x0F
        ANDWF   S0,W
        IORWF   T0,W
        CALL    I2CWB

        MOVF    M1,W    ; MINUTES
        MOVWF   T0
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        MOVLW   0xF0
        ANDWF   T0,F
        MOVLW   0x0F
        ANDWF   M0,W
        IORWF   T0,W
        CALL    I2CWB

        MOVF    H1,W    ; HOURS
        MOVWF   T0
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        RLF     T0,F
        MOVLW   0xF0
        ANDWF   T0,F
        MOVLW   0x0F
        ANDWF   H0,W
        IORWF   T0,W
        ANDLW   0x3F
        CALL    I2CWB

        CALL    I2CSTO

	RETURN



; ******************************************************************
; *** READ TIME FROM RTC
; ******************************************************************
RTIME:  CALL    I2CSTA
        MOVLW   0xA0    ; address
        CALL    I2CWB
        MOVLW   0x01    ; 01 = HS/SS/MM/HH
        CALL    I2CWB
        CALL    I2CSTO

        CALL    I2CSTA
        MOVLW   0xA1
        CALL    I2CWB
        CALL    I2CRB	
;        MOVWF   HS	; DISCARD 100's SECONDS
        CALL    I2CRB
        MOVWF   S0
        CALL    I2CRB
        MOVWF   M0
        CALL    I2CRL   ; READ LAST = NO I2CSTO NEEDED
        MOVWF   H0

        ; TAKE CARE OF TIME DATA
	; CONVERT 1-BYTE TO 2-NIBBLES (10's & 1's)
        MOVF    S0,W
        MOVWF   S1
        MOVLW   0x0F
        ANDWF   S0,F
        RRF     S1,F
        RRF     S1,F
        RRF     S1,F
        RRF     S1,F
        MOVLW   0x0F
        ANDWF   S1,F

        MOVF    M0,W
        MOVWF   M1
        MOVLW   0x0F
        ANDWF   M0,F
        RRF     M1,F
        RRF     M1,F
        RRF     M1,F
        RRF     M1,F
        MOVLW   0x0F
        ANDWF   M1,F

        MOVF    H0,W
        MOVWF   H1
        MOVLW   0x0F
        ANDWF   H0,F
        RRF     H1,F
        RRF     H1,F
        RRF     H1,F
        RRF     H1,F
        MOVLW   0x0F
        ANDWF   H1,F
	RETURN




; ******************************************************************
; *** MAIN PROGRAM BEGINS HERE!!!
; ******************************************************************
Start:	; MAIN PROGRAM INITIALIZATION POINT

        CALL    INIT
	CALL	I2CINIT

        CALL    I2CSTO

        call    D500ms

        CALL    I2CSTA          ; INITIALIZE 8583 CONFIG WORD
        MOVLW   0xA0
        CALL    I2CWB
        MOVLW   0x00
        CALL    I2CWB
        MOVLW   0x00
        CALL    I2CWB
        CALL    I2CSTO

	CALL	D500ms
	CALL	D500ms
	CALL	D500ms


	MOVLW	0x04		; INITIALIZE SOME REGISTERS
	MOVWF	MPX
	CLRF	FLASH
	CLRF	FLCNT
	CLRF	MODE
	CLRF	BDEB
	BSF	NEN		; NIXIE MASTER ENABLE. 1 = SHREG OUTPUTS ENABLED


MPX0:	; MULTIPLEX LOOP
	MOVF	MODE,F		; IF NOT MODE=0, DON'T READ TIME FROM RTC (DURING SET)
	BTFSC	STATUS,Z
	CALL	RTIME		; IF MODE=0, READ TIME FROM RTS

	MOVF	MODE,F		; IF NOT MODE=0, DELAY FOR A WHILE
	BTFSS	STATUS,Z
	CALL	D10ms

	INCF	FLCNT,F		; INCREASE FLASH COUNTER EACH LOOP

	; CHECK BUTTONS
CKMNU:	BTFSC	B_MNU		; CHECK IF MENU-BUTTON IS PRESSED
	GOTO	CKPLU
	BTFSC	BDEB,0		; IF NOT RELEASED SINCE LAST TURN, SKIP!
	GOTO	BTNDONE
	BSF	BDEB,0		; REMEMBER BUTTON PRESSED FOR DEBOUNCE
	; TAKE CARE OF MODE-BUTTON
	INCF	MODE,F		; NEXT MODE (SET NEXT DIGIT)
	MOVF	MODE,W		; CHECK IF SET IS DONE
	SUBLW	.7
	BTFSS	STATUS,Z
	GOTO 	BTNDONE
	BCF	X0
	BCF	X1
	BCF	X2
	CALL	WTIME
	CLRF	MODE		; DONE, RETURN TO NORMAL DISPLAY
	GOTO	BTNDONE


CKPLU:	BTFSC	B_PL		; CHECK IF PLUS-BUTTON IS PRESSED
	GOTO	CKMIN
	BTFSC	BDEB,1
	GOTO	BTNDONE
	BSF	BDEB,1
	; TAKE CARE OF PLUS-BUTTON

	MOVF	MODE,F		; NO INCREASE IF MODE = 0 (CLOCK DISPLAY)
	BTFSC	STATUS,Z
	GOTO	BTNDONE

	MOVF	MODE,W		; POINT IND TO 0x24 + MODE
	ADDLW	0x24
	MOVWF	FSR

	INCF	INDF,F		; INCREASE SELECTED DIGIT BY 1

PLWR1:	MOVF	MODE,W		; CHECK WRAP MODE=1 (0..2)
	SUBLW	.1
	BTFSS	STATUS,Z
	GOTO	PLWR2
	MOVF	INDF,W
	SUBLW	.3
	BTFSC	STATUS,Z
	CLRF	INDF

PLWR2:	MOVF	MODE,W		; CHECK WRAP MODE=2 (0..9,0..3)
	SUBLW	.2
	BTFSS	STATUS,Z
	GOTO	PLWR3

	DECF	FSR,F		; IF 10xHOUR = 2 THEN WRAP 1xHOUR 0..3
	MOVF	INDF,W
	INCF	FSR,F
	SUBLW	.2
	BTFSS	STATUS,Z
	GOTO	PLWR2a
	MOVF	INDF,W
	SUBLW	.4
	BTFSC	STATUS,Z
	CLRF	INDF
PLWR2a:	MOVF	INDF,W
	SUBLW	.10
	BTFSC	STATUS,Z
	CLRF	INDF

PLWR3:	MOVF	MODE,W		; CHECK WRAP MODE=3 (0..5)
	SUBLW	.3
	BTFSS	STATUS,Z
	GOTO	PLWR4
	MOVF	INDF,W
	SUBLW	.6
	BTFSC	STATUS,Z
	CLRF	INDF

PLWR4:	MOVF	MODE,W		; CHECK WRAP MODE=4 (0..9)
	SUBLW	.4
	BTFSS	STATUS,Z
	GOTO	PLWR5
	MOVF	INDF,W
	SUBLW	.10
	BTFSC	STATUS,Z
	CLRF	INDF

PLWR5:	MOVF	MODE,W		; CHECK WRAP MODE=5 (0..5)
	SUBLW	.5
	BTFSS	STATUS,Z
	GOTO	PLWR6
	MOVF	INDF,W
	SUBLW	.6
	BTFSC	STATUS,Z
	CLRF	INDF

PLWR6:	MOVF	MODE,W		; CHECK WRAP MODE=6 (0..9)
	SUBLW	.6
	BTFSS	STATUS,Z
	GOTO	BTNDONE
	MOVF	INDF,W
	SUBLW	.10
	BTFSC	STATUS,Z
	CLRF	INDF

	GOTO	BTNDONE		; ALL MODES ARE NOW WRAP-CHECKED!


CKMIN:	BTFSC	B_MI		; CHECK IF MINUS-BUTTON IS PRESSED
	GOTO	NOBTN
	BTFSC	BDEB,2
	GOTO	BTNDONE
	BSF	BDEB,2
	; TAKE CARE OF MINUS-BUTTON

	MOVF	MODE,F		; NO DECREASE IF MODE = 0 (CLOCK DISPLAY)
	BTFSC	STATUS,Z
	GOTO	BTNDONE

	MOVF	MODE,W		; POINT IND TO 0x24 + MODE
	ADDLW	0x24
	MOVWF	FSR

	MOVF	INDF,F		; IF INDF=0 -> WRAP, RELOAD
	BTFSC	STATUS,Z
	GOTO	MIWR1

	DECF	INDF,F		; DECREASE SELECTED DIGIT BY 1
	GOTO	BTNDONE
	
MIWR1:	MOVF	MODE,W		; CHECK WRAP MODE=0 (2..0)
	SUBLW	.1
	BTFSS	STATUS,Z
	GOTO	MIWR2
	MOVLW	.2
	MOVWF	INDF

MIWR2:	MOVF	MODE,W		; CHECK WRAP MODE=1 (9..0,3..0)
	SUBLW	.2
	BTFSS	STATUS,Z
	GOTO	MIWR3

	DECF	FSR,F		; IF 10xHOUR=2 THEN WRAP TO 3
	MOVF	INDF,W
	INCF	FSR,F
	SUBLW	.2
	BTFSS	STATUS,Z
	GOTO	MIWR2a
	MOVLW	.3
	MOVWF	INDF
	GOTO	MIWR3
MIWR2a:	MOVLW	.9		; IF 10xHOUR=0..1 THEN WRAP TO 9
	MOVWF	INDF

MIWR3:	MOVF	MODE,W		; CHECK WRAP MODE=3 (5..0)
	SUBLW	.3
	BTFSS	STATUS,Z
	GOTO	MIWR4
	MOVLW	.5
	MOVWF	INDF

MIWR4:	MOVF	MODE,W		; CHECK WRAP MODE=4 (9..0)
	SUBLW	.4
	BTFSS	STATUS,Z
	GOTO	MIWR5
	MOVLW	.9
	MOVWF	INDF

MIWR5:	MOVF	MODE,W		; CHECK WRAP MODE=5 (5..0)
	SUBLW	.5
	BTFSS	STATUS,Z
	GOTO	MIWR6
	MOVLW	.5
	MOVWF	INDF

MIWR6:	MOVF	MODE,W		; CHECK WRAP MODE=6 (9..0)
	SUBLW	.6
	BTFSS	STATUS,Z
	GOTO	BTNDONE
	MOVLW	.9
	MOVWF	INDF

	GOTO	BTNDONE		; ALL MODES IS NOW WRAP-CHECKED!


NOBTN:	CLRF	BDEB		; NO BUTTONS PRESSED, RESET DEBOUNCE-FLAG
BTNDONE:

	; DISPLAY CURSOR?
	CLRF	FLASH
	MOVF	MODE,W
	BTFSC	STATUS,Z
	GOTO	CRSDONE

	MOVWF	_N		; TRANSLATE MODE 1-6 TO FLASH[0..5]
	BSF	STATUS,C
CRSCNT:	RLF	FLASH,F
	BCF	STATUS,C
	DECFSZ	_N,F
	GOTO	CRSCNT
CRSDONE:

	; FLASH 10-DIGIT!?
	BCF	FLASH,7
	BTFSS	FLCNT,6		; FLASH WITH 50%-DUTYCYCLE (USE BIT 6 IN COUNTER)
	GOTO	SND1		; NO FLASH THIS TIME

FLH1:	BTFSS	MPX,2		; ARE WE GOING TO DISPLAY HOURS?
	GOTO	FLM1
	BTFSS	FLASH,0
	GOTO	FLM1
	BSF	FLASH,7

FLM1:	BTFSS	MPX,1		; ARE WE GOING TO DISPLAY MINUTES?
	GOTO	FLS1
	BTFSS	FLASH,2
	GOTO	FLS1
	BSF	FLASH,7
	
FLS1:	BTFSS	MPX,0		; ARE WE GOING TO DISPLAY SECONDS?
	GOTO	SND1
	BTFSS	FLASH,4
	GOTO	SND1
	BSF	FLASH,7

	; NOW SEND 10-DIGIT TO SHREG
SND1:	MOVLW	.6		; 10-DIGITS CONSISTS OF 0..5 = 6 DIGITS
	MOVWF	_N
MPX1:	BCF	NDA
	; SELECT DATA TO DISPLAY
	BTFSC	MPX,2
	MOVF	H1,W
	BTFSC	MPX,1
	MOVF	M1,W
	BTFSC	MPX,0
	MOVF	S1,W
	; SHOULD THIS BIT BE 1 OR 0?
	SUBWF	_N,W
	SUBLW	1
	BTFSC	STATUS,Z
	BSF	NDA
	; FLASH?
	BTFSC	FLASH,7
	BCF	NDA
	; CLOCK BIT TO SHREG
	NOP		
	BSF	NCK
	BCF	NCK
	DECFSZ	_N,F
	GOTO	MPX1	


	; FLASH 1-DIGIT!?
	BCF	FLASH,7
	BTFSS	FLCNT,6
	GOTO	SND0

FLH0:	BTFSS	MPX,2
	GOTO	FLM0
	BTFSS	FLASH,1
	GOTO	FLM0
	BSF	FLASH,7

FLM0:	BTFSS	MPX,1
	GOTO	FLS0
	BTFSS	FLASH,3
	GOTO	FLS0
	BSF	FLASH,7
	
FLS0:	BTFSS	MPX,0
	GOTO	SND0
	BTFSS	FLASH,5
	GOTO	SND0
	BSF	FLASH,7

	; NOW SEND 1-DIGIT TO SHREG
SND0:	MOVLW	.10		; 1-DIGITS CONSISTS OF 0..9 = 10 DIGITS
	MOVWF	_N
MPX2:	BCF	NDA
	; SELECT DATA To DISPLAY
	BTFSC	MPX,2
	MOVF	H0,W
	BTFSC	MPX,1
	MOVF	M0,W
	BTFSC	MPX,0
	MOVF	S0,W
	; SHOULD THIS BIT BE 1 OR 0?
	SUBWF	_N,W
	SUBLW	1
	BTFSC	STATUS,Z
	BSF	NDA
	; FLASH?
	BTFSC	FLASH,7
	BCF	NDA
	; CLOCK BIT TO SHREG
	NOP		
	BSF	NCK
	BCF	NCK
	DECFSZ	_N,F
	GOTO	MPX2	


	; SHUT DOWN ALL NIXIES
	BCF	X0
	BCF	X1
	BCF	X2

	; STROBE REGISTER
	BSF	NST		; OK, SKICKA EN STROBE TILL DRIVKORTEN
	NOP
	BCF	NST

	; LIGHT UP THE RIGHT GROUP OF NIXIES  (X0, X1 or X2)
	BTFSC	MPX,2
	BSF	X2
	BTFSC	MPX,1
	BSF	X1
	BTFSC	MPX,0
	BSF	X0

	; MOVE MULTIPLEX POINTER 2->1->0->2->...
	RRF	MPX,F
	BTFSS	STATUS,C
	GOTO	MPX0
	MOVLW	0x04	; RESET POINTER
	MOVWF	MPX
	GOTO	MPX0


ende	goto	ende







;****************************************************************************             
;*** Modules
;****************************************************************************            


	include "_delay.asm"
	include "_i2c.asm"
;	include "_misc.asm"


;
;****************************************************************************
;****************************************************************************
;****************************************************************************

        END
